/* Copyright (c) 2009 Arnaud Renevier, Inc, published under the modified BSD
 * license. */

OpenLayers.Control.SypAttribution = OpenLayers.Class (OpenLayers.Control.Attribution, {
    updateAttribution: function() {
        var attributions = [SypStrings.propulsedByLink];
        if (this.map && this.map.layers) {
            for(var i=0, len=this.map.layers.length; i<len; i++) {
                var layer = this.map.layers[i];
                if (layer.attribution && layer.getVisibility()) {
                    attributions.push( layer.attribution );
                }
            }  
            this.div.innerHTML = attributions.join(this.separator);
        }
    }
});

var SYP = {
    Settings: {
        MARKER_ICON: "openlayers/img/marker-blue.png",
        MARKER_ICON_HEIGHT: 25,
        MARKER_SELECT_ICON: "openlayers/img/marker-green.png",
        MARKER_SELECT_ICON_HEIGHT: 25
    },

    map: null,
    baseLayer: null,
    dataLayer: null,
    selectControl: null,

    init: function() {
        this.map = new OpenLayers.Map("map", {
            controls:[
                new OpenLayers.Control.Navigation(),
                new OpenLayers.Control.PanZoom(),
                new OpenLayers.Control.Permalink(),
                new OpenLayers.Control.SypAttribution()
            ],
            projection: new OpenLayers.Projection("EPSG:900913"),
            displayProjection: new OpenLayers.Projection("EPSG:4326")
        } );

        this.baseLayer = this.createBaseLayer();
        this.dataLayer = this.createDataLayer();
        this.map.addLayers([this.baseLayer, this.dataLayer]);

        this.selectControl = this.createSelectControl();
        this.map.addControl(this.selectControl);
        this.selectControl.activate();

        if (!this.map.getCenter()) {
            var centerBounds = new OpenLayers.Bounds();

            var mapProj = this.map.getProjectionObject();
            var sypOrigProj = new OpenLayers.Projection("EPSG:4326");

            var bottomLeft = new OpenLayers.LonLat(sypOrig[0],sypOrig[1]);
            bottomLeft = bottomLeft.transform(sypOrigProj, mapProj);
            var topRight = new OpenLayers.LonLat(sypOrig[2],sypOrig[3])
            topRight = topRight.transform(sypOrigProj, mapProj);

            centerBounds.extend(bottomLeft);
            centerBounds.extend(topRight);
            this.map.zoomToExtent(centerBounds);
        }
    },

    createBaseLayer: function() {
        return new OpenLayers.Layer.OSM("OSM");
    },

    createDataLayer: function(map) {
        var styleMap = new OpenLayers.StyleMap (
                        {"default": {
                            externalGraphic: this.Settings.MARKER_ICON,
                            graphicHeight: this.Settings.MARKER_ICON_HEIGHT
                                            || 32 
                                    },
                         "select": { 
                            externalGraphic: this.Settings.MARKER_SELECT_ICON,
                            graphicHeight: this.Settings.MARKER_SELECT_ICON_HEIGHT 
                                            || 32 
                                  }
                     });

        var layer = new OpenLayers.Layer.GML("KML", "items.php", 
           {
            styleMap: styleMap,
            format: OpenLayers.Format.KML, 
            projection: this.map.displayProjection,
            eventListeners: { scope: this,
                              loadend: this.checkForFeatures
                            }
           });

        return layer;
    },

    createSelectControl: function() {
        var control = new OpenLayers.Control.SelectFeature(
                                            this.dataLayer, {
                                               onSelect: this.onFeatureSelect,
                                               onUnselect: this.onFeatureUnselect,
                                               toggle: true,
                                               clickout: false
                                                            });
        return control;
    },

    checkForFeatures: function() {
        var features = this.dataLayer.features;
        if (features.length == 0) {
            var message = SypStrings.noImageRegistered;
            this.Utils.displayUserMessage(message, "warn");
        }
    },

    createPopup: function(position, contentHTML) {
        var popup = new OpenLayers.Popup.Anchored("popup",
                                                  position,
                                                  null,
                                                  contentHTML,
                                                  null,
                                                  true);
        popup.autoSize = true;
        popup.backgroundColor = ""; // deal with it in css
        popup.border = ""; // deal with it in css
        popup.closeOnMove = true;
        return popup;
    },

    onFeatureUnselect: function (feature) {
        var map = feature.layer.map;
        var permaControl = map.getControlsByClass("OpenLayers.Control.Permalink");
        if (permaControl[0]) {
            permaControl[0].div.style.display = "";
        }
        var popup = feature.popup;
        if (popup.visible()) {
            popup.hide();
        }
    },

    onFeatureSelect: function(feature) {
        var map = feature.layer.map;
        var permaControl = map.getControlsByClass("OpenLayers.Control.Permalink");
        if (permaControl[0]) {
            permaControl[0].div.style.display = "none";
        }
        var popup = feature.popup;

        var brCorner = SYP.Utils.brCorner(map, 8);

        // we cannot reuse popup; we need to recreate it in order for IE
        // expressions to work. Otherwise, we get a 0x0 image on second view.
        if (popup) { 
            popup.destroy();
        }
        var contentHTML;
        if (feature.attributes.name) {
            contentHTML = "<h2>" +
                          feature.attributes.name + 
                          "</h2>" + 
                          feature.attributes.description;
        } else {
            contentHTML = feature.attributes.description;
        }
        popup = SYP.createPopup(brCorner, contentHTML);
        var control = this;
        popup.hide = function () {
            OpenLayers.Element.hide(this.div);
            control.unselect(feature);
        };
        map.addPopup(popup);
        feature.popup = popup;
    },

    Utils: {
        brCorner: function(map, margin) {
            var bounds = map.calculateBounds();
            var corner = new OpenLayers.LonLat(bounds.right, bounds.bottom);
            var cornerAsPx = map.getPixelFromLonLat(corner);
            cornerAsPx = cornerAsPx.add( -margin, -margin);
            corner = map.getLonLatFromPixel(cornerAsPx);
            return corner;
        },
        displayUserMessage: function(message, status) {
            var div = document.getElementById('message');
            while (div.firstChild)
                div.removeChild(div.firstChild);
            var textNode = document.createTextNode(message);
            switch (status) {
                case "error":
                    div.style.color = "red";
                    break;
                case "warn":
                    div.style.color = "#FF8C00";
                    break;
                case "success":
                    div.style.color = "green";
                    break;
                default:
                    div.style.color = "black";
                    break;
            }
            div.style.display = "block";
            div.appendChild(textNode);
        }
    }
};

// if possible, determine language with HTTP_ACCEPT_LANGUAGE instead of
// navigator.language
if (OpenLayers.Lang[SypStrings.language]) {
    OpenLayers.Lang.setCode(SypStrings.language);
}

// avoid alerts
OpenLayers.Console.userError = function(error) { 
    SYP.Utils.displayUserMessage(error, "error");
}
